<?php

/**
 * @encoding     UTF-8
 * @package      MSP - MultiSport & eSport WordPress plugin with 90 Visual Composer addons
 * @link         https://codecanyon.net/item/msp-multisport-esport-wordpress-plugin-with-90-visual-composer-addons/20954667
 * @copyright    Copyright (C) 2018 Torbara (http://torbara.com). All rights reserved.
 * @license      Envato Standard License http://themeforest.net/licenses/standard?ref=torbara
 * @author       Vladislav Dolgolenko <vladislavdolgolenko.com>
 * @support      <help@vladislavdolgolenko.com>
 */

namespace MSP\Model;

use MSP\Model\AbstractPostModel as AbstractModel;
use MSP\DataType\ModelType;
use MSP\Model\MatchWeather as Weather;
use MSP\WPObjects\Date\WordPress as WPDate;

class Match extends AbstractModel implements
    \MSP\DataType\MatchStatus
{
    /**
     * @var \MSP\Model\MatchWeather
     */
    protected $Weather = null;
    
    static public function getSortingTypes()
    {
        return array(
            __('Date start', 'msp') => '_date_start',
            __('Date publ', 'msp') => 'date',
        );
    }
    
    static public function getStatusesForCheckbox()
    {
        return array(
            __('TBD', 'msp') => self::TBD,
            __('Postponed', 'msp') => self::POSTPONNED,
            __('On time', 'msp') => self::EXPECTED,
            __('live', 'msp') => self::LIVE,
            __('finished', 'msp') => self::FINISHED,
            __('Canceled', 'msp') => self::CANCELED,
        );
    }
    
    public function getStatusName()
    {
        $match_status_id = $this->getStatus();
        switch ($match_status_id){
            case self::TBD:
                return __('TBD', 'msp');
            case self::POSTPONNED:
                return __('Postponed', 'msp');
            case self::EXPECTED:
                return __('On time', 'msp');
            case self::LIVE:
                return __('live', 'msp');
            case self::FINISHED:
                return __('finished', 'msp');
            case self::CANCELED:
                return __('Canceled', 'msp');
        }

        return __('N/A', 'msp');
    }
    
    public function getTitleForSelect()
    {
        $team_left = $this->getTeamByNumber(0);
        $team_right = $this->getTeamByNumber(1);
        $team_left_stat = $this->getTeamMainScore($team_left->getId());
        $team_right_stat = $this->getTeamMainScore($team_right->getId());
        
        $title = $team_left->getName();
        $title .= " [$team_left_stat:$team_right_stat] ";
        $title .= $team_right->getName();
        
        return $title;
    }
    
    public function getImgForSelect()
    {
        return '';
    }
    
    /**
     * @return int match status datatype
     */
    public function getStatus()
    {
        $time_status = $this->getTimeStatus();
        if ($time_status) {
            return $time_status;
        }

        $timestart = $this->getTimeStart();
        if ($timestart > time() ) {
            return self::EXPECTED;
        }

        $team_winner_id = $this->getTeamWinnerId();
        if ($team_winner_id) {
            return self::FINISHED;
        }

        return self::LIVE;
    }
    
    public function getTimeStatus()
    {
        return $this->getMeta('_time_status');
    }

    public function getTimeStartJSFormat()
    {
        return $this->getTimeStart() . "000";
    }
    
    public function getTimeStart()
    {
        $date_start = $this->getTimeStartFormat('Y-m-d H:i:s T');
        if (!$date_start) {
            return 0;
        }

        return strtotime($date_start);
    }
    
    /**
    * @param string time format
    * @return string with time
    */
    public function getTimeStartFormat($format = 'j F Y / G:i T')
    {
        $date_start = $this->getDateStart();
        return WPDate::formatFromString($date_start, $format);
    }
    
    public function getTimeLength()
    {
        if ($this->getStatus() < 1) {
            return false;
        }

        $date_start = $this->getDateStart();
        $date_end = $this->getDateEnd();
        if (!$date_start) {
            return false;
        }

        $time_start = strtotime($date_start);
        $time_end = strtotime($date_end);
        if (!$time_end || $time_start > $time_end) {
            return 90;
        }

        return $time_end - $time_start;
    }
    
    public function getDateStart()
    {
        return $this->getMeta('_date_start');
    }
    
    public function setDateStart($date)
    {
        if (strtotime($date) > 0) {
            $this->setMeta('_date_start', $date);
        }
        
        return $this;
    }
    
    public function getDateEnd()
    {
        return $this->getMeta('_date_end');
    }
    
    public function setDateEnd($date)
    {
        if (strtotime($date) > 0) {
            $this->setMeta('_date_end', $date);
        }
        
        return $this;
    }
    
    /**
     * @return \MSP\Model\Venue
     */
    public function getVenue()
    {
        return $this->getRelative(ModelType::VENUE);
    }
    
    /**
     * @return \MSP\Model\Game
     */
    public function getGame()
    {
        return $this->getRelative(ModelType::GAME);
    }
    
    public function getGameId()
    {
        return $this->getRelativeId(ModelType::GAME);
    }
    
    public function getTeamMainScore($team_id)
    {
        $game_id = $this->getGameId();
        $StatFactory = $this->getServiceManager()->get('TeamStatisticTypeFactory');
        $Stat = $StatFactory->getMainStat($game_id);
        if (!$Stat) {
            return '';
        }
        
        return $this->getTeamData($team_id, $Stat->getId());
    }
    
    public function getTeamBansHeroes($team_id)
    {
        $heroes_ids = $this->getTeamData($team_id, '_dota_bans_heroes');
        if (!is_array($heroes_ids)) {
            return array();
        }
        
        return $this->getServiceManager()->get('PlayerHeroFactory')->get($heroes_ids, array(), false);
    }
    
    public function getTeamById($id)
    {
        $teams = $this->getTeams();
        foreach ($teams as $Team) {
            if ($Team->getId() == $id) {
                return $Team;
            }
        }
        
        return null;
    }
    
    public function isTeamWinner($team_id)
    {
        $team_winner_id = $this->getTeamWinnerId();
    
        if ($team_winner_id === "draw") {
            return "draw";
        }

        if ($team_winner_id == $team_id) {
            return true;
        }

        return false;
    }
    
    /**
     * @return \MSP\Model\Team
     */
    public function getTeams()
    {
        return $this->getRelative(ModelType::TEAM, array(), false);
    }
    
    /**
     * @return \MSP\Model\Team
     */
    public function getTeamByNumber($number = 0)
    {
        $teams = $this->getTeams();
        return isset($teams[$number]) ? $teams[$number] : null;
    }
    
    public function getTeamApponent($team_id)
    {
        $Teams = $this->getTeams();
        foreach ($Teams as $Team) {
            if ($Team->getId() != $team_id) {
                return $Team;
            }
        }
        
        return null;
    }
    
    public function getTeamWinnerId()
    {
        return $this->getMeta('_team_winner_id');
    }
    
    public function setTeamWinnerId($id)
    {
        $this->setMeta('_team_winner_id', $id);
        
        return $this;
    }
    
    public function isPlayerPlaying($player_id)
    {
        $ModelTypePlayer = $this->getServiceManager()->get('ModelTypeFactory')->get(ModelType::PLAYER);
        $team_attr = $ModelTypePlayer->getQualifierAttrName(ModelType::PLAYER_TEAM_ACTIVITY);
        $player_activity_id = $this->getPlayerData($player_id, $team_attr);
        if ($player_activity_id === 'primary_staff' || 
            $player_activity_id === 'participated'
        ) {
            return true;
        }
        
        return false;
    }
    
    public function isPlayerWinner($player_id)
    {
        $team_winner_id = $this->getTeamWinnerId();
        $ModelTypePlayer = $this->getServiceManager()->get('ModelTypeFactory')->get(ModelType::PLAYER);
        $team_attr = $ModelTypePlayer->getQualifierAttrName(ModelType::TEAM);
        $player_team_id = $this->getPlayerData($player_id, $team_attr);

        if ($team_winner_id === "draw") {
            return "draw";
        }

        if ((int)$team_winner_id === (int)$player_team_id) {
            return true;
        }

        return false;
    }
    
    public function getPlayerTeam($player_id)
    {
        $ModelTypePlayer = $this->getServiceManager()->get('ModelTypeFactory')->get(ModelType::PLAYER);
        $team_attr = $ModelTypePlayer->getQualifierAttrName(ModelType::TEAM);
        $team_id = $this->getPlayerData($player_id, $team_attr);
        return $this->getTeamById($team_id);
    }
    
    public function getPlayerById($player_id)
    {
        $players = $this->getPlayers();
        foreach ($players as $Player) {
            if ($Player->getId() == $player_id) {
                return $Player;
            }
        }
        
        return null;
    }
    
    /**
     * @return \MSP\Model\Player
     */
    public function getPlayers($filter = array())
    {
        return $this->getRelative(ModelType::PLAYER, $filter, false);
    }
    
    /**
     * @return \MSP\Model\Player
     */
    public function getPlayersByMatchData($filters = array(), $query_filters = array())
    {
        $Players = $this->getPlayers($query_filters);
        
        $register_filters = array(
            '_organisation_id',
            '_player_position_id',
            '_player_team_activity_id',
            '_player_capitan'
        );
        
        $result = array();
        foreach ($Players as $Player) {
            
            $confirm = true;
            foreach ($filters as $filter_key => $filter_value) {
                if (!in_array($filter_key, $register_filters) || $confirm === false || $filter_value === "") {
                    continue;
                }
                
                if ($filter_value === "true") {
                    $filter_value = 1;
                }
                
                $value = $this->getPlayerData($Player->getId(), $filter_key);
                if ($value != $filter_value) {
                    $confirm = false;
                }
            }
            
            if ($confirm === true) {
                $result[] = $Player;
            }
        }
        
        return $result;
    }
    
    /**
     * @return \MSP\Model\Player
     */
    public function getPlayersByTeam($filter_team_id)
    {
        $result = array();
        $ids = $this->getPlayersIdsByTeam($filter_team_id);
        foreach ($this->getPlayers() as $Player) {
            if (in_array($Player->getId(), $ids)) {
                $result[] = $Player;
            }
        }
        
        return $result;
    }
    
    public function getPlayersIdsByTeam($filter_team_id)
    {
        $result = array();
        foreach ($this->getPlayers() as $Player) {
            $ModelTypePlayer = $this->getServiceManager()->get('ModelTypePlayer');
            $team_attr = $ModelTypePlayer->getQualifierAttrName(ModelType::TEAM);
            $team_id = $this->getPlayerData($Player->getId(), $team_attr);
            if ($team_id == $filter_team_id) {
                $result[] = $Player->getId();
            }
        }
        
        return array_unique($result);
    }
    
    public function getPlayersIds()
    {
        $attr = $this->getModelType()->getQualifierAttrName(ModelType::PLAYER);
        $meta = $this->getMeta($attr);
        if (!is_array($meta)) {
            return array($meta);
        }
        
        return $meta;
    }
    
    public function getPlayerPositions()
    {
        $Game = $this->getGame(); 
        if (!$Game) {
            return array();
        }
        
        return $Game->getPlayerPositions();
    }
    
    public function getPlayerHero($player_id)
    {
        $hero_id = $this->getPlayerData($player_id, '_player_hero_id');
        if (!$hero_id) {
            return null;
        }
        
        $PlayerHeroFactory = $this->getServiceManager()->get('PlayerHeroFactory');
        return $PlayerHeroFactory->get($hero_id);
    }
    
    public function getPlayerItems($player_id)
    {
        $items_ids = $this->getPlayerData($player_id, '_player_items');
        if (!is_array($items_ids) || !$items_ids) {
            return array();
        }
        
        $Factory = $this->getServiceManager()->get('PlayerItemFactory');
        return $Factory->query(array('id' => $items_ids))->getResult();
    }

    public function getPlayerSpells($player_id)
    {
        $attr_name = $this->getServiceManager()->get('ModelTypePlayer')->getQualifierAttrName(ModelType::LOL_PLAYER_SPELL);
        $spells_ids = $this->getPlayerData($player_id, $attr_name);

        $SpellFactory = $this->getServiceManager()->get('LOLPlayerSpellFactory');
        return $SpellFactory->query(array('id' => $spells_ids))->getResult();
    }
    
    public function getPlayersGroupByPositions()
    {
        $team_left = $this->getTeamByNumber(0); 
        $result = array(
            'left' => array(),
            'right' => array()
        );

        $positions = $this->getPlayerPositions();
        foreach ($positions as $position) {
            $result['right'][$position->id] = array();
            $result['left'][$position->id] = array();
        }

        $players = $this->getPlayers();
        foreach ($players as $player) {
            if ($this->getPlayerData($player->ID, '_player_team_activity_id') != "primary_staff") {
                continue;
            }

            $position_id = $this->getPlayerData($player->ID, '_player_position_id');
            if (!$position_id) {
                continue;
            }

            if ($this->getPlayerData($player->ID, '_organisation_id') == $team_left->ID) {
                $result['left'][$position_id][] = $player;
            } else {
                $result['right'][$position_id][] = $player;
            }

        }

        return $result;
    }
    
    public function getTrainersByTeam($team_id)
    {
        $trainers_ids = $this->getTrainersIdsByTeam($team_id);
        if (!count($trainers_ids)) {
            return array();
        }
        
        $Factory = $this->getServiceManager()->get('TrainerFactory');
        return $Factory->query(array('id' => $trainers_ids))->getResult();
    }
    
    /**
     * @param int || string $team_id
     * @return array
     */
    public function getTrainersIdsByTeam($team_id)
    {
        $ModelType = $this->getServiceManager()->get('ModelTypeTeam');
        $trainer_attr = $ModelType->getQualifierAttrName(ModelType::TRAINER);
        $trainers_ids = $this->getTeamData($team_id, $trainer_attr);
        return is_array($trainers_ids) ? $trainers_ids : array($trainers_ids);
    }
    
    
    public function updateTrainers($trainer_ids, $team_id)
    {
        if (!$team_id || !is_array($trainer_ids)) {
            return;
        }
        
        $data = $this->getTeamData($team_id);
        $attr = $this->getServiceManager()->get('ModelTypeTeam')->getQualifierAttrName(ModelType::TRAINER);
        $data[$attr] = $trainer_ids;
        
        $this->setTeamData($team_id, $data);
        
        return $this;
    }
    
    public function getReferees()
    {
        return $this->getRelative(ModelType::REFEREE, array(), false);
    }
    
    public function getChampionship()
    {
        return $this->getRelative(ModelType::CHAMPIONSHIP);
    }
    
    public function getBroadcasts()
    {
        return $this->getRelative(ModelType::BROADCAST, array(),  false);
    }
    
    public function getBroadcastsIds()
    {
        return $this->getRelativeId(ModelType::BROADCAST, false);
    }
    
    /**
     * @return \MSP\Model\MatchWeather
     */
    public function getWeather()
    {
        if (is_null($this->Weather)) {
            $this->Weather = new Weather(array(
                'ID' => $this->getMeta('_weather_type_id'),
                'degrees' => $this->getMeta('_weather_degrees'),
                'description' => $this->getMeta('_weather_description'),
            ));
        }
        
        return $this->Weather;
    }
    
    public function getLiveTimeUpdate()
    {
        $time = (int)$this->getMeta('_live_update_time') * 1000;
        if ($time < 1000) {
            return 1000;
        }
        
        return $time;
    }
    
    public function isLiveUpdated()
    {
        return $this->getMeta('_live_update') ? true : false;
    }
    
    /**
     * @return array
     */
    public function getPlayerData($player_id, $key = null)
    {
        $data = $this->getMeta($this->getPlayerMatchDataKey($player_id));
        if (!is_array($data)) {
            $data = array();
        }

        if (!$key) {
            return $data;
        }

        if (isset($data[$key])) {
            return $data[$key];
        }

        return '';
    }
    
    static public function getPlayerMatchDataKey($player_id)
    {
        return '_player_data_' . $player_id;
    }
    
    public function getTeamData($team_id, $key = null)
    {
        $data = $this->getMeta($this->getTeamMatchDataKey($team_id));
        if (!is_array($data)) {
            $data = array();
        }

        if (!$key) {
            return $data;
        }

        if (isset($data[$key])) {
            return $data[$key];
        }

        return '';
    }
    
    public function setTeamData($team_id, $data)
    {
        $this->setMeta($this->getTeamMatchDataKey($team_id), $data);
        
        return $this;
    }
    
    static public function getTeamMatchDataKey($team_id)
    {
        return '_organisation_data_' . $team_id;
    }
    
    public function setPlayersids($ids)
    {
        $attr = $this->getModelType()->getQualifierAttrName(ModelType::PLAYER);
        $this->setMeta($attr, $ids);
        
        return $this;
    }
    
    
    public function updatePlayers($new_players_ids, $organisation_id)
    {
        if (!$organisation_id || !is_array($new_players_ids)) {
            return;
        }

        $current_player_ids = $this->getPlayersIdsByTeam($organisation_id);

        $delete_ids = array_unique( array_diff( $current_player_ids, $new_players_ids ) );
        $insert_ids = array_unique( array_diff( $new_players_ids, $current_player_ids ) );

        foreach ($delete_ids as $id) {
            $this->removePlayer($id);
        }

        foreach ($insert_ids as $id) {
            $this->addPlayer($id, $organisation_id);
        }
    }
    
    public function addPlayer($player_id, $team_id)
    {
        // Добавляем идемнтификатор
        $players_ids = $this->getPlayersIds();
        $index = array_search((string)$player_id, $players_ids);
        if ($index === false) {
            $players_ids[] = $player_id;
        }
        $this->setPlayersids($players_ids);
        
        // Копируем данные
        $Player = $this->getServiceManager()->get('PlayerFactory')->get($player_id);
        $this->updatePlayerMatchData($player_id, $Player->getDataForMatch($team_id));
        
        return $this;
    }
    
    public function removePlayer($player_id)
    {
        // Удаляем идентификатор
        $players_ids = $this->getPlayersIds();
        $index = array_search((string)$player_id, $players_ids);
        if ($index !== false) {
            unset($players_ids[$index]);
        }
        $this->setPlayersids($players_ids);
        
        // Удаляем данные
        $this->removePlayerMatchData($player_id);
        
        return $this;
    }
    
    public function updatePlayerMatchData($player_id, $new_data)
    {
        $data_attr = $this->getPlayerMatchDataKey($player_id);
        $current_data = $this->getMeta($data_attr);
        if (!is_array($current_data)) {
            $current_data = array();
        }
        
        $this->setMeta($data_attr, array_merge($current_data, $new_data));
        
        return $this;
    }
    
    public function removePlayerMatchData($player_id)
    {
        $data_attr = $this->getPlayerMatchDataKey($player_id);
        $this->setMeta($data_attr, null);
        
        return $this;
    }
    
    /**
     * League of Legends match system identity obtained via Riot api
     * 
     * @return string|int
     */
    public function getLOLId()
    {
        return $this->getMeta('_lol_id');
    }
    
    /**
     * League of Legends match system identity obtained via Riot api
     * 
     * @param int\string $id
     * @return $this
     */
    public function setLOLId($id)
    {
        $this->setMeta('_lol_id', $id);
        
        return $this;
    }
    
    /**
     * League of Legends system match all data obtained via Riot api
     * 
     * @return array
     */
    public function getLOLData()
    {
        return $this->getMeta('_lol_data');
    }
    
    /**
     * League of Legends system match all data obtained via Riot api
     * 
     * @param array $data
     * @return $this
     */
    public function setLOLData($data)
    {
        $this->setMeta('_lol_data', $data);
        
        return $this;
    }
}